@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2015 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,15 +20,16 @@
 #define PHP_XDEBUG_H
 
 #define XDEBUG_NAME       "Xdebug"
-#define XDEBUG_VERSION    "2.3.3"
+#define XDEBUG_VERSION    "2.4.0"
 #define XDEBUG_AUTHOR     "Derick Rethans"
-#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2015 by Derick Rethans"
-#define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2015"
+#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2016 by Derick Rethans"
+#define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2016"
 #define XDEBUG_URL        "http://xdebug.org"
 #define XDEBUG_URL_FAQ    "http://xdebug.org/docs/faq#api"
 
 #include "php.h"
 
+#include "xdebug_compat.h"
 #include "xdebug_handlers.h"
 #include "xdebug_hash.h"
 #include "xdebug_llist.h"
@@ -104,6 +105,11 @@ PHP_FUNCTION(xdebug_start_error_collection);
 PHP_FUNCTION(xdebug_stop_error_collection);
 PHP_FUNCTION(xdebug_get_collected_errors);
 
+/* function monitorin functions */
+PHP_FUNCTION(xdebug_start_function_monitor);
+PHP_FUNCTION(xdebug_stop_function_monitor);
+PHP_FUNCTION(xdebug_get_monitored_functions);
+
 /* profiling functions */
 PHP_FUNCTION(xdebug_get_profiler_filename);
 PHP_FUNCTION(xdebug_dump_aggr_profiling_data);
@@ -120,18 +126,19 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	int           status;
 	int           reason;
 
-	long          level;
+	unsigned long level;
 	xdebug_llist *stack;
-	long          max_nesting_level;
-	long          max_stack_frames;
+	iniLONG       max_nesting_level;
+	iniLONG       max_stack_frames;
 	zend_bool     default_enable;
 	zend_bool     collect_includes;
-	long          collect_params;
+	iniLONG       collect_params;
 	zend_bool     collect_return;
 	zend_bool     collect_vars;
 	zend_bool     collect_assignments;
 	zend_bool     extended_info;
 	zend_bool     show_ex_trace;
+	zend_bool     show_error_trace;
 	zend_bool     show_local_vars;
 	zend_bool     show_mem_delta;
 	double        start_time;
@@ -142,8 +149,8 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	unsigned int  prev_memory;
 	char         *file_link_format;
 	zend_bool     force_display_errors;
-	long          force_error_reporting;
-	long          halt_level;
+	iniLONG       force_error_reporting;
+	iniLONG       halt_level;
 
 	zend_bool     overload_var_dump;
 	void        (*orig_var_dump_func)(INTERNAL_FUNCTION_PARAMETERS);
@@ -158,16 +165,16 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	char         *trace_enable_trigger_value;
 	char         *trace_output_dir;
 	char         *trace_output_name;
-	long          trace_options;
-	//char         *tracefile_name;
-	int           trace_format;
+	iniLONG       trace_options;
+	iniLONG       trace_format;
 	char         *last_exception_trace;
 	char         *last_eval_statement;
+	zend_bool     in_debug_info;
 
 	/* variable dumping limitation settings */
-	long          display_max_children;
-	long          display_max_data;
-	long          display_max_depth;
+	iniLONG       display_max_children;
+	iniLONG       display_max_data;
+	iniLONG       display_max_depth;
 
 	zend_bool     cli_color;
 	int           output_is_tty;
@@ -189,7 +196,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	xdebug_path_info     *paths_stack;
 	xdebug_hash          *visited_branches;
 	struct {
-		int  size;
+		unsigned int  size;
 		int *last_branch_nr;
 	} branches;
 
@@ -197,6 +204,11 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	zend_bool     do_collect_errors;
 	xdebug_llist *collected_errors;
 
+	/* used for function monitoring */
+	zend_bool     do_monitor_functions;
+	xdebug_hash  *functions_to_monitor;
+	xdebug_llist *monitored_functions_found; /* List of functions found */
+
 	/* superglobals */
 	zend_bool     dump_globals;
 	zend_bool     dump_once;
@@ -216,7 +228,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 
 	/* remote settings */
 	zend_bool     remote_enable;  /* 0 */
-	long          remote_port;    /* 9000 */
+	iniLONG       remote_port;    /* 9000 */
 	char         *remote_host;    /* localhost */
 	long          remote_mode;    /* XDEBUG_NONE, XDEBUG_JIT, XDEBUG_REQ */
 	char         *remote_handler; /* php3, gdb, dbgp */
@@ -224,7 +236,8 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	zend_bool     remote_connect_back;   /* connect back to the HTTP requestor */
 	char         *remote_log;       /* Filename to log protocol communication to */
 	FILE         *remote_log_file;  /* File handler for protocol log */
-	long          remote_cookie_expire_time; /* Expire time for the remote-session cookie */
+	iniLONG       remote_cookie_expire_time; /* Expire time for the remote-session cookie */
+	char         *remote_addr_header; /* User configured header to check for forwarded IP address */
 
 	char         *ide_key; /* As Xdebug uses it, from environment, USER, USERNAME or empty */
 	char         *ide_key_setting; /* Set through php.ini and friends */
