@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -180,6 +180,20 @@ char* xdebug_memnstr(char *haystack, char *needle, int needle_len, char *end)
 	return NULL;
 }
 
+char* xdebug_strrstr(const char* haystack, const char* needle)
+{
+	char  *loc = NULL;
+	char  *found = NULL;
+	size_t pos = 0;
+
+	while ((found = strstr(haystack + pos, needle)) != 0) {
+		loc = found;
+		pos = (found - haystack) + 1;
+	}
+
+	return loc;
+}
+
 double xdebug_get_utime(void)
 {
 #ifdef HAVE_GETTIMEOFDAY
@@ -320,12 +334,6 @@ char *xdebug_path_from_url(const char *fileurl TSRMLS_DC)
 	return ret;
 }
 
-#if PHP_VERSION_ID >= 50400
-# define VIRTUAL_FILE_EX_TSRMLS_CC TSRMLS_CC
-#else
-# define VIRTUAL_FILE_EX_TSRMLS_CC
-#endif
-
 /* fake URI's per IETF RFC 1738 and 2396 format */
 char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 {
@@ -350,15 +358,23 @@ char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 			cwd[0] = '\0';
 		}
 
+#if PHP_VERSION_ID >= 50600
+		new_state.cwd = estrdup(cwd);
+#else
 		new_state.cwd = strdup(cwd);
+#endif
 		new_state.cwd_length = strlen(cwd);
 
-		if (!virtual_file_ex(&new_state, fileurl, NULL, 1 VIRTUAL_FILE_EX_TSRMLS_CC)) {
+		if (!virtual_file_ex(&new_state, fileurl, NULL, 1 TSRMLS_CC)) {
 			char *s = estrndup(new_state.cwd, new_state.cwd_length);
 			tmp = xdebug_sprintf("file://%s",s);
 			efree(s);
 		}
+#if PHP_VERSION_ID >= 50600
+		efree(new_state.cwd);
+#else
 		free(new_state.cwd);
+#endif
 
 	} else if (fileurl[1] == '/' || fileurl[1] == '\\') {
 		/* convert UNC paths (eg. \\server\sharepath) */
@@ -530,7 +546,7 @@ FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname)
 
 int xdebug_format_output_filename(char **filename, char *format, char *script_name)
 {
-	xdebug_str fname = {0, 0, NULL};
+	xdebug_str fname = XDEBUG_STR_INITIALIZER;
 	char       cwd[128];
 	TSRMLS_FETCH();
 	
@@ -588,7 +604,7 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				}	break;
 
 				case 'u': { /* timestamp (in microseconds) */
-					char *char_ptr, *utime = xdebug_sprintf("%f", xdebug_get_utime());
+					char *char_ptr, *utime = xdebug_sprintf("%F", xdebug_get_utime());
 					
 					/* Replace . with _ (or should it be nuked?) */
 					char_ptr = strrchr(utime, '.');  
@@ -601,9 +617,28 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				case 'H':   /* $_SERVER['HTTP_HOST'] */
 				case 'U':   /* $_SERVER['UNIQUE_ID'] */
 				case 'R': { /* $_SERVER['REQUEST_URI'] */
-					zval **data;
 					char *char_ptr, *strval;
+#if PHP_VERSION_ID >= 70000
+					zval *data = NULL;
+
+					if (Z_TYPE(PG(http_globals)[TRACK_VARS_SERVER]) == IS_ARRAY) {
+						switch (*format) {
+						case 'H':
+							data = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "HTTP_HOST", sizeof("HTTP_HOST") - 1);
+							break;
+						case 'R':
+							data = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "REQUEST_URI", sizeof("REQUEST_URI") - 1);
+							break;
+						case 'U':
+							data = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "UNIQUE_ID", sizeof("UNIQUE_ID") - 1);
+							break;
+						}
+
+						if (data) {
+							strval = estrdup(Z_STRVAL_P(data));
+#else
 					int retval = FAILURE;
+					zval **data;
 
 					if (PG(http_globals)[TRACK_VARS_SERVER]) {
 						switch (*format) {
@@ -620,6 +655,7 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 
 						if (retval == SUCCESS) {
 							strval = estrdup(Z_STRVAL_PP(data));
+#endif
 							
 							/* replace slashes, dots, question marks, plus
 							 * signs, ampersands, spaces and other evil chars
@@ -634,18 +670,29 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				}	break;
 
 				case 'S': { /* session id */
+#if PHP_VERSION_ID >= 70000
+					zval *data;
+#else
 					zval **data;
+#endif
 					char *char_ptr, *strval;
 					char *sess_name;
 					
 					sess_name = zend_ini_string("session.name", sizeof("session.name"), 0);
 					
+#if PHP_VERSION_ID >= 70000
+					if (sess_name && Z_TYPE(PG(http_globals)[TRACK_VARS_COOKIE]) == IS_ARRAY &&
+						((data = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_COOKIE]), sess_name, strlen(sess_name))) != NULL) &&
+						Z_STRLEN_P(data) < 100 /* Prevent any unrealistically long data being set as filename */
+					) {
+						strval = estrdup(Z_STRVAL_P(data));
+#else
 					if (sess_name && PG(http_globals)[TRACK_VARS_COOKIE] &&
 						zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_COOKIE]), sess_name, strlen(sess_name) + 1, (void **) &data) == SUCCESS &&
 						Z_STRLEN_PP(data) < 100 /* Prevent any unrealistically long data being set as filename */
 					) {
 						strval = estrdup(Z_STRVAL_PP(data));
-						
+#endif
 						/* replace slashes, dots, question marks, plus signs,
 						 * ampersands and spaces with underscores */
 						while ((char_ptr = strpbrk(strval, "/\\.?&+ ")) != NULL) {
@@ -669,6 +716,40 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 	return fname.l;
 }
 
+int xdebug_format_file_link(char **filename, const char *error_filename, int error_lineno TSRMLS_DC)
+{
+	xdebug_str fname = XDEBUG_STR_INITIALIZER;
+	char      *format = XG(file_link_format);
+
+	while (*format)
+	{
+		if (*format != '%') {
+			xdebug_str_addl(&fname, (char *) format, 1, 0);
+		} else {
+			format++;
+			switch (*format)
+			{
+				case 'f': /* filename */
+					xdebug_str_add(&fname, xdebug_sprintf("%s", error_filename), 1);
+					break;
+
+				case 'l': /* line number */
+					xdebug_str_add(&fname, xdebug_sprintf("%d", error_lineno), 1);
+					break;
+
+				case '%': /* literal % */
+					xdebug_str_addl(&fname, "%", 1, 0);
+					break;
+			}
+		}
+		format++;
+	}
+
+	*filename = fname.d;
+
+	return fname.l;
+}
+
 void xdebug_open_log(TSRMLS_D)
 {
 	/* initialize remote log file */
@@ -681,6 +762,8 @@ void xdebug_open_log(TSRMLS_D)
 		fprintf(XG(remote_log_file), "Log opened at %s\n", timestr);
 		fflush(XG(remote_log_file));
 		xdfree(timestr);
+	} else if (strlen(XG(remote_log))) {
+		php_log_err(xdebug_sprintf("XDebug could not open the remote debug file '%s'.", XG(remote_log)) TSRMLS_CC);
 	}
 }
 
