@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2015 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,13 +15,15 @@
    | Authors:  Derick Rethans <derick@xdebug.org>                         |
    +----------------------------------------------------------------------+
  */
-
-
+#include "php_xdebug.h"
 #include <string.h>
 #include <sys/types.h>
 #include <errno.h>
 #include <stdio.h>
 #include <fcntl.h>
+#if HAVE_SYS_SELECT_H
+# include <sys/select.h>
+#endif
 #ifndef PHP_WIN32
 #include <unistd.h>
 #endif
@@ -126,13 +128,16 @@ int xdebug_create_socket(const char *hostname, int dport)
 #ifdef WIN32
 		errno = WSAGetLastError();
 		if (errno != WSAEINPROGRESS && errno != WSAEWOULDBLOCK) {
+			close(sockfd);
 			return -1;
 		}
 #else
 		if (errno == EACCES) {
+			close(sockfd);
 			return -3;
 		}
 		if (errno != EINPROGRESS) {
+			close(sockfd);
 			return -1;
 		}
 #endif
@@ -148,11 +153,13 @@ int xdebug_create_socket(const char *hostname, int dport)
 			FD_SET(sockfd, &eset);
 
 			if (select(sockfd+1, &rset, &wset, &eset, &timeout) == 0) {
+				close(sockfd);
 				return -2;
 			}
 
 			/* if our descriptor has an error */
 			if (FD_ISSET(sockfd, &eset)) {
+				close(sockfd);
 				return -1;
 			}
 
@@ -164,6 +171,7 @@ int xdebug_create_socket(const char *hostname, int dport)
 
 		actually_connected = getpeername(sockfd, &sa, &size);
 		if (actually_connected == -1) {
+			close(sockfd);
 			return -1;
 		}
 	}
