@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -86,7 +86,7 @@ int xdebug_profiler_init(char *script_name TSRMLS_DC)
 	if (XG(profiler_append)) {
 		fprintf(XG(profile_file), "\n==== NEW PROFILING FILE ==============================================\n");
 	}
-	fprintf(XG(profile_file), "version: 1\ncreator: xdebug %s\n", XDEBUG_VERSION);
+	fprintf(XG(profile_file), "version: 1\ncreator: xdebug %s (PHP %s)\n", XDEBUG_VERSION, PHP_VERSION);
 	fprintf(XG(profile_file), "cmd: %s\npart: 1\npositions: line\n\n", script_name);
 	fprintf(XG(profile_file), "events: Time\n\n");
 	fflush(XG(profile_file));
@@ -131,6 +131,31 @@ void xdebug_profiler_function_user_begin(function_stack_entry *fse TSRMLS_DC)
 	fse->profile.mark = xdebug_get_utime();
 }
 
+static char* get_filename_ref(char *name TSRMLS_DC)
+{
+	long nr;
+
+	if (xdebug_hash_find(XG(profile_filename_refs), name, strlen(name), (void*) &nr)) {
+		return xdebug_sprintf("(%d)", nr);
+	} else {
+		XG(profile_last_filename_ref)++;
+		xdebug_hash_add(XG(profile_filename_refs), name, strlen(name), (void*) (size_t) XG(profile_last_filename_ref));
+		return xdebug_sprintf("(%d) %s", XG(profile_last_filename_ref), name);
+	}
+}
+
+static char* get_functionname_ref(char *name TSRMLS_DC)
+{
+	long nr;
+
+	if (xdebug_hash_find(XG(profile_functionname_refs), name, strlen(name), (void*) &nr)) {
+		return xdebug_sprintf("(%d)", nr);
+	} else {
+		XG(profile_last_functionname_ref)++;
+		xdebug_hash_add(XG(profile_functionname_refs), name, strlen(name), (void*) (size_t) XG(profile_last_functionname_ref));
+		return xdebug_sprintf("(%d) %s", XG(profile_last_functionname_ref), name);
+	}
+}
 
 void xdebug_profiler_function_user_end(function_stack_entry *fse, zend_op_array* op_array TSRMLS_DC)
 {
@@ -171,7 +196,7 @@ void xdebug_profiler_function_user_end(function_stack_entry *fse, zend_op_array*
 
 	if (fse->prev) {
 		xdebug_call_entry *ce = xdmalloc(sizeof(xdebug_call_entry));
-		ce->filename = op_array ? xdstrdup(op_array->filename) : xdstrdup(fse->filename);
+		ce->filename = op_array ? xdstrdup(STR_NAME_VAL(op_array->filename)) : xdstrdup(fse->filename);
 		ce->function = xdstrdup(tmp_name);
 		ce->time_taken = fse->profile.time;
 		ce->lineno = fse->lineno;
@@ -181,15 +206,28 @@ void xdebug_profiler_function_user_end(function_stack_entry *fse, zend_op_array*
 	}
 
 	if (fse->user_defined == XDEBUG_EXTERNAL) {
+		char *tmp_fl = NULL, *tmp_fn = NULL;
 		if (op_array) {
-			fprintf(XG(profile_file), "fl=%s\n", op_array->filename);
+			tmp_fl = get_filename_ref((char*) STR_NAME_VAL(op_array->filename) TSRMLS_CC);
+			fprintf(XG(profile_file), "fl=%s\n", tmp_fl);
 		} else {
-			fprintf(XG(profile_file), "fl=%s\n", fse->filename);
+			tmp_fl = get_filename_ref(fse->filename TSRMLS_CC);
+			fprintf(XG(profile_file), "fl=%s\n", tmp_fl);
 		}
-		fprintf(XG(profile_file), "fn=%s\n", tmp_name);
+		tmp_fn = get_functionname_ref(tmp_name TSRMLS_CC);
+		fprintf(XG(profile_file), "fn=%s\n", tmp_fn);
+
+		xdfree(tmp_fl);
+		xdfree(tmp_fn);
 	} else {
-		fprintf(XG(profile_file), "fl=php:internal\n");
-		fprintf(XG(profile_file), "fn=php::%s\n", tmp_name);
+		char *tmp_key = xdebug_sprintf("php::%s", tmp_name);
+		char *tmp_fl = get_filename_ref("php:internal" TSRMLS_CC);
+		char *tmp_fn = get_functionname_ref(tmp_key TSRMLS_CC);
+		fprintf(XG(profile_file), "fl=%s\n", tmp_fl);
+		fprintf(XG(profile_file), "fn=%s\n", tmp_fn);
+		xdfree(tmp_fl);
+		xdfree(tmp_fn);
+		xdfree(tmp_key);
 	}
 	xdfree(tmp_name);
 
@@ -223,11 +261,22 @@ void xdebug_profiler_function_user_end(function_stack_entry *fse, zend_op_array*
 		xdebug_call_entry *call_entry = XDEBUG_LLIST_VALP(le);
 
 		if (call_entry->user_defined == XDEBUG_EXTERNAL) {
-			fprintf(XG(profile_file), "cfl=%s\n", call_entry->filename);
-			fprintf(XG(profile_file), "cfn=%s\n", call_entry->function);
+			char *tmp_fl = NULL, *tmp_fn = NULL;
+			tmp_fl = get_filename_ref(call_entry->filename TSRMLS_CC);
+			tmp_fn = get_functionname_ref(call_entry->function TSRMLS_CC);
+			fprintf(XG(profile_file), "cfl=%s\n", tmp_fl);
+			fprintf(XG(profile_file), "cfn=%s\n", tmp_fn);
+			xdfree(tmp_fl);
+			xdfree(tmp_fn);
 		} else {
-			fprintf(XG(profile_file), "cfl=php:internal\n");
-			fprintf(XG(profile_file), "cfn=php::%s\n", call_entry->function);
+			char *tmp_key = xdebug_sprintf("php::%s", call_entry->function);
+			char *tmp_fl = get_filename_ref("php:internal" TSRMLS_CC);
+			char *tmp_fn = get_functionname_ref(tmp_key TSRMLS_CC);
+			fprintf(XG(profile_file), "cfl=%s\n", tmp_fl);
+			fprintf(XG(profile_file), "cfn=%s\n", tmp_fn);
+			xdfree(tmp_fl);
+			xdfree(tmp_fn);
+			xdfree(tmp_key);
 		}
 		
 		fprintf(XG(profile_file), "calls=1 0 0\n");
@@ -249,7 +298,11 @@ void xdebug_profiler_function_internal_end(function_stack_entry *fse TSRMLS_DC)
 	xdebug_profiler_function_user_end(fse, NULL TSRMLS_CC);
 }
 
+#if PHP_VERSION_ID >= 70000
+static int xdebug_print_aggr_entry(zval *pDest, void *argument TSRMLS_DC)
+#else
 static int xdebug_print_aggr_entry(void *pDest, void *argument TSRMLS_DC)
+#endif
 {
 	FILE *fp = (FILE *) argument;
 	xdebug_aggregate_entry *xae = (xdebug_aggregate_entry *) pDest;
@@ -261,6 +314,15 @@ static int xdebug_print_aggr_entry(void *pDest, void *argument TSRMLS_DC)
 		fprintf(fp, "\nsummary: %lu\n\n", (unsigned long) (xae->time_inclusive * 1000000));
 	}
 	if (xae->call_list) {
+#if PHP_VERSION_ID >= 70000
+		xdebug_aggregate_entry *xae_call;
+
+		ZEND_HASH_FOREACH_PTR(xae->call_list, xae_call) {
+			fprintf(fp, "cfn=%s\n", (xae_call)->function);
+			fprintf(fp, "calls=%d 0 0\n", (xae_call)->call_count);
+			fprintf(fp, "%d %lu\n", (xae_call)->lineno, (unsigned long) ((xae_call)->time_inclusive * 1000000));
+		} ZEND_HASH_FOREACH_END();
+#else
 		xdebug_aggregate_entry **xae_call;
 
 		zend_hash_internal_pointer_reset(xae->call_list);
@@ -270,6 +332,7 @@ static int xdebug_print_aggr_entry(void *pDest, void *argument TSRMLS_DC)
 			fprintf(fp, "%d %lu\n", (*xae_call)->lineno, (unsigned long) ((*xae_call)->time_inclusive * 1000000));
 			zend_hash_move_forward(xae->call_list);
 		}
+#endif
 	}
 	fprintf(fp, "\n");
 	fflush(fp);
