@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2015 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -69,6 +69,10 @@ static void dump_hash_elem(zval *z, char *name, long index, char *elem, int html
 	}
 }
 
+#if PHP_VERSION_ID >= 70000
+static int dump_hash_elem_va(zval *pDest, zend_ulong index, zend_string *hash_key, char *name, int html, xdebug_str *str)
+{
+#else
 static int dump_hash_elem_va(void *pDest TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	int html;
@@ -78,11 +82,16 @@ static int dump_hash_elem_va(void *pDest TSRMLS_DC, int num_args, va_list args,
 	name = va_arg(args, char *);
 	html = va_arg(args, int);
 	str =  va_arg(args, xdebug_str *);
+#endif
 
-	if (hash_key->nKeyLength == 0) {
+	if (HASH_KEY_IS_NUMERIC(hash_key)) {
 		dump_hash_elem(*((zval **) pDest), name, hash_key->h, NULL, html, str TSRMLS_CC);
 	} else {
+#if PHP_VERSION_ID >= 70000
+		dump_hash_elem(pDest, name, 0, HASH_APPLY_KEY_VAL(hash_key), html, str TSRMLS_CC);
+#else
 		dump_hash_elem(*((zval **) pDest), name, 0, (char *) hash_key->arKey, html, str TSRMLS_CC);
+#endif
 	}
 
 	return SUCCESS;
@@ -90,7 +99,14 @@ static int dump_hash_elem_va(void *pDest TSRMLS_DC, int num_args, va_list args,
 
 static void dump_hash(xdebug_llist *l, char *name, int name_len, int html, xdebug_str *str TSRMLS_DC)
 {
+#if PHP_VERSION_ID >= 70000
+	zval *z;
+	zend_ulong num;
+	zend_string *key;
+	zval *val;
+#else
 	zval **z;
+#endif
 	HashTable *ht = NULL;
 	xdebug_llist_element *elem;
 
@@ -98,11 +114,26 @@ static void dump_hash(xdebug_llist *l, char *name, int name_len, int html, xdebu
 		return;
 	}
 
+#if PHP_VERSION_ID >= 70000
+	{
+		zend_string *s_name = zend_string_init(name, name_len, 0);
+		if ((z = zend_hash_find(&EG(symbol_table), s_name))) {
+			if (Z_TYPE_P(z) == IS_REFERENCE) {
+				z = &z->value.ref->val;
+			}
+			if (Z_TYPE_P(z) == IS_ARRAY) {
+				ht = Z_ARRVAL_P(z);
+			}
+		}
+		zend_string_release(s_name);
+	}
+#else
 	if (zend_hash_find(&EG(symbol_table), name, name_len, (void **) &z) == SUCCESS) {
 		if (Z_TYPE_PP(z) == IS_ARRAY) {
 			ht = Z_ARRVAL_PP(z);
 		}
 	}
+#endif
 
 	if (html) {
 		xdebug_str_add(str, xdebug_sprintf("<tr><th colspan='5' align='left' bgcolor='#e9b96e'>Dump <i>$%s</i></th></tr>\n", name), 1);
@@ -113,30 +144,48 @@ static void dump_hash(xdebug_llist *l, char *name, int name_len, int html, xdebu
 	elem = XDEBUG_LLIST_HEAD(l);
 
 	while (elem != NULL) {
+#if PHP_VERSION_ID >= 70000
+		zend_string *s;
+
+		s = zend_string_init(elem->ptr, strlen(elem->ptr), 0);
+#endif
+
 		if (ht && (*((char *) (elem->ptr)) == '*')) {
+#if PHP_VERSION_ID >= 70000
+			ZEND_HASH_FOREACH_KEY_VAL_IND(ht, num, key, val) {
+				dump_hash_elem_va(val, num, key, name, html, str);
+			} ZEND_HASH_FOREACH_END();
+		} else if (ht && (z = zend_hash_find(ht, s))) {
+			dump_hash_elem(z, name, 0, elem->ptr, html, str TSRMLS_CC);
+#else
 			zend_hash_apply_with_arguments(ht TSRMLS_CC, dump_hash_elem_va, 3, name, html, str);
 		} else if (ht && zend_hash_find(ht, elem->ptr, strlen(elem->ptr) + 1, (void **) &z) == SUCCESS) {
 			dump_hash_elem(*z, name, 0, elem->ptr, html, str TSRMLS_CC);
+#endif
 		} else if(XG(dump_undefined)) {
 			dump_hash_elem(NULL, name, 0, elem->ptr, html, str TSRMLS_CC);
 		}
 
 		elem = XDEBUG_LLIST_NEXT(elem);
+
+#if PHP_VERSION_ID >= 70000
+		zend_string_release(s);
+#endif
 	}
 }
 
 char* xdebug_get_printable_superglobals(int html TSRMLS_DC)
 {
-	xdebug_str str = {0, 0, NULL};
+	xdebug_str str = XDEBUG_STR_INITIALIZER;
 
-	dump_hash(&XG(server),  "_SERVER",  8, html, &str TSRMLS_CC);
-	dump_hash(&XG(get),     "_GET",     5, html, &str TSRMLS_CC);
-	dump_hash(&XG(post),    "_POST",    6, html, &str TSRMLS_CC);
-	dump_hash(&XG(cookie),  "_COOKIE",  8, html, &str TSRMLS_CC);
-	dump_hash(&XG(files),   "_FILES",   7, html, &str TSRMLS_CC);
-	dump_hash(&XG(env),     "_ENV",     5, html, &str TSRMLS_CC);
-	dump_hash(&XG(session), "_SESSION", 9, html, &str TSRMLS_CC);
-	dump_hash(&XG(request), "_REQUEST", 9, html, &str TSRMLS_CC);
+	dump_hash(&XG(server),  "_SERVER",  HASH_KEY_SIZEOF("_SERVER"),  html, &str TSRMLS_CC);
+	dump_hash(&XG(get),     "_GET",     HASH_KEY_SIZEOF("_GET"),     html, &str TSRMLS_CC);
+	dump_hash(&XG(post),    "_POST",    HASH_KEY_SIZEOF("_POST"),    html, &str TSRMLS_CC);
+	dump_hash(&XG(cookie),  "_COOKIE",  HASH_KEY_SIZEOF("_COOKIE"),  html, &str TSRMLS_CC);
+	dump_hash(&XG(files),   "_FILES",   HASH_KEY_SIZEOF("_FILES"),   html, &str TSRMLS_CC);
+	dump_hash(&XG(env),     "_ENV",     HASH_KEY_SIZEOF("_ENV"),     html, &str TSRMLS_CC);
+	dump_hash(&XG(session), "_SESSION", HASH_KEY_SIZEOF("_SESSION"), html, &str TSRMLS_CC);
+	dump_hash(&XG(request), "_REQUEST", HASH_KEY_SIZEOF("_REQUEST"), html, &str TSRMLS_CC);
 
 	return str.d;
 }
